@@ -32,7 +32,6 @@ from firewall import config
 from firewall.dbus_utils import dbus_to_python, \
     dbus_introspection_prepare_properties, \
     dbus_introspection_add_properties
-from firewall.core.io.service import Service
 from firewall.core.logger import log
 from firewall.server.decorators import handle_exceptions, \
     dbus_handle_exceptions, dbus_service_method
@@ -173,7 +172,7 @@ class FirewallDConfigService(slip.dbus.service.Object):
     # S E T T I N G S
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
-                         out_signature=Service.DBUS_SIGNATURE)
+                         out_signature='(sssa(ss)asa{ss}asa(ss))')
     @dbus_handle_exceptions
     def getSettings(self, sender=None): # pylint: disable=W0613
         """get settings for service
@@ -182,7 +181,16 @@ class FirewallDConfigService(slip.dbus.service.Object):
         return self.config.get_service_config(self.obj)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
-                         in_signature=Service.DBUS_SIGNATURE)
+                         out_signature='a{sv}')
+    @dbus_handle_exceptions
+    def getSettings2(self, sender=None):
+        """get settings for service
+        """
+        log.debug1("%s.getSettings2()", self._log_prefix)
+        return self.config.get_service_config_dict(self.obj)
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='(sssa(ss)asa{ss}asa(ss))')
     @dbus_handle_exceptions
     def update(self, settings, sender=None):
         """update settings for service
@@ -193,6 +201,16 @@ class FirewallDConfigService(slip.dbus.service.Object):
         self.obj = self.config.set_service_config(self.obj, settings)
         self.Updated(self.obj.name)
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='a{sv}')
+    @dbus_handle_exceptions
+    def update2(self, settings, sender=None):
+        settings = dbus_to_python(settings)
+        log.debug1("%s.update2('...')", self._log_prefix)
+        self.parent.accessCheck(sender)
+        self.obj = self.config.set_service_config_dict(self.obj, settings)
+        self.Updated(self.obj.name)
+
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE)
     @dbus_handle_exceptions
     def loadDefaults(self, sender=None):
@@ -652,3 +670,64 @@ class FirewallDConfigService(slip.dbus.service.Object):
         settings = self.getSettings()
         return (family in settings[5] and
                 address == settings[5][family])
+
+    # includes
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         out_signature='as')
+    @dbus_handle_exceptions
+    def getIncludes(self, sender=None):
+        log.debug1("%s.getIncludes()", self._log_prefix)
+        self.parent.accessCheck(sender)
+        settings = list(self.config.get_service_config(self.obj))
+        return settings[8]
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='as')
+    @dbus_handle_exceptions
+    def setIncludes(self, includes, sender=None):
+        includes = dbus_to_python(includes, list)
+        log.debug1("%s.setIncludes('%s')", self._log_prefix, includes)
+        self.parent.accessCheck(sender)
+        settings = list(self.config.get_service_config(self.obj))
+        settings[8] = includes[:]
+        self.config.set_service_config(self.obj, tuple(settings))
+        self.Updated(self.obj.name)
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='s')
+    @dbus_handle_exceptions
+    def addInclude(self, include, sender=None):
+        include = dbus_to_python(include, str)
+        log.debug1("%s.addInclude('%s')", self._log_prefix, include)
+        self.parent.accessCheck(sender)
+        settings = list(self.config.get_service_config(self.obj))
+        settings[8].append(include)
+        self.config.set_service_config(self.obj, tuple(settings))
+        self.Updated(self.obj.name)
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='s')
+    @dbus_handle_exceptions
+    def removeInclude(self, include, sender=None):
+        include = dbus_to_python(include, str)
+        log.debug1("%s.removeInclude('%s')", self._log_prefix, include)
+        self.parent.accessCheck(sender)
+        settings = list(self.config.get_service_config(self.obj))
+        settings[8].remove(include)
+        self.config.set_service_config(self.obj, tuple(settings))
+        self.Updated(self.obj.name)
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='s', out_signature='b')
+    @dbus_handle_exceptions
+    def queryInclude(self, include, sender=None):
+        include = dbus_to_python(include, str)
+        log.debug1("%s.queryInclude('%s')", self._log_prefix, include)
+        settings = list(self.config.get_service_config(self.obj))
+        try:
+            settings[8].index(include)
+        except ValueError:
+            return False
+        else:
+            return True
